package PureMessage::Install::Prefix;
use strict;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw($PREFIX read_prefix_file $prefix_file);

our $prefix_file;
our $PREFIX;

# look at each of these files
my @prefix_files = qw(/etc/puremessage-6);
unshift @prefix_files, $ENV{PMX_PREFIX_FILE} unless $> == 0;

foreach my $f (@prefix_files) {
    next unless defined $f and -e $f;

    open(my $fh, $f) or die "Can't open $f: $!";
    unless (-e $fh) {
        die "Can't determine PureMessage install prefix: $f doesn't exist!\n";
    }

    # The file must either be owned by us, or owned by root
    unless (-o _ || (stat(_))[4] == 0) {
        my $ename = getpwuid $>;
        die "Error - $f is not owned by us ($ename) or root - "
            . "Can't determine PureMessage install prefix.\n";
    }

    # The file must not be writeable by other users
    if ((stat(_))[2] & 022) {
        die "Error - $f is writeable by other users! - "
            . "Can't determine PureMessage install prefix.\n";
    }

    eval {
        $PREFIX = read_prefix_file($fh);
    };
    if ($@) {
        warn $@;
        $PREFIX = undef;
        next;
    }
    close $fh or die "Can't close $f: $!";
    next unless defined $PREFIX;

    $prefix_file = $f;
    last;
}

die "No PureMessage install prefix file exists! (@prefix_files)\n" 
    unless defined $prefix_file;

die "The PureMessage install prefix cannot be found!\n" unless defined $PREFIX;

1;

sub read_prefix_file {
    my $fh;
    my $opened;
    my $prefix;
    if (ref($_[0]) eq "GLOB") {
        $fh = $_[0];
    }
    else {
        open($fh, $_[0]) or die "Can't open $_[0]: $!";
        $opened++;
    }

    # use the last line in the file and remove trailing slash and newline 
    while(<$fh>) {
        chomp;
        $prefix = $_ if -d $_;
    }
    if ($prefix) {
        $prefix =~ s,/$,,;
	# if running as root, check that the prefix is owned by root
        $prefix = undef if $> == 0 and -d $prefix and (stat(_))[4] != 0;
    }

    if ($opened) {
        close $fh or warn "Couldn't close $_[0]: $!";
    }
    return $prefix;
}

__END__

=head1 NAME

PureMessage::Install::Prefix;

=head1 SYNOPSIS

  use PureMessage::Install::Prefix qw($PREFIX);

=head1 DESCRIPTION

This module determines the prefix of the currently installed PureMessage.

It does not use the PureMessage::Config module, and has no further 
dependencies.

=head1 PUBLIC FUNCTIONS

=head2 read_prefix_file

This function takes either an open filehandle or a file name and returns the 
prefix from this file.

=head1 CONFIGURATION

By default, the /etc/puremessage-6 file will be used to determine the 
PureMessage install prefix.  This can be overridden with the 

  $PMX_PREFIX_FILE 

environment variable.  This variable specifies a file that contains the 
PureMessage prefix.  The specified file must be owned by the current user
for security reasons.

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2004, Sophos PLC
All Rights Reserved.



